@@ -14,6 +14,7 @@
 
 import contextlib
 import inspect
+import jmespath
 import munch
 import netifaces
 import re
@@ -59,7 +60,7 @@ def _iterate_timeout(timeout, message, wait=2):
     while (timeout is None) or (time.time() < start + timeout):
         count += 1
         yield count
-        log.debug('Waiting {wait} seconds'.format(wait=wait))
+        log.debug('Waiting %s seconds', wait)
         time.sleep(wait)
     raise exc.OpenStackCloudTimeout(message)
 
@@ -73,7 +74,7 @@ def _filter_list(data, name_or_id, filters):
         key if a value for name_or_id is given.
     :param string name_or_id:
         The name or ID of the entity being filtered.
-    :param dict filters:
+    :param filters:
         A dictionary of meta data to use for further filtering. Elements
         of this dictionary may, themselves, be dictionaries. Example::
 
@@ -83,6 +84,8 @@ def _filter_list(data, name_or_id, filters):
                   'gender': 'Female'
               }
             }
+        OR
+        A string containing a jmespath expression for further filtering.
     """
     if name_or_id:
         identifier_matches = []
@@ -96,6 +99,9 @@ def _filter_list(data, name_or_id, filters):
     if not filters:
         return data
 
+    if isinstance(filters, six.string_types):
+        return jmespath.search(filters, data)
+
     def _dict_filter(f, d):
         if not d:
             return False
@@ -129,8 +135,11 @@ def _get_entity(func, name_or_id, filters, **kwargs):
         and returns a list of entities to filter.
     :param string name_or_id:
         The name or ID of the entity being filtered or a dict
-    :param dict filters:
+    :param filters:
         A dictionary of meta data to use for further filtering.
+        OR
+        A string containing a jmespath expression for further filtering.
+        Example:: "[?last_name==`Smith`] | [?other.gender]==`Female`]"
     """
     # Sometimes in the control flow of shade, we already have an object
     # fetched. Rather than then needing to pull the name or id out of that
@@ -148,36 +157,6 @@ def _get_entity(func, name_or_id, filters, **kwargs):
     return entities[0]
 
 
-def normalize_servers(servers, cloud_name, region_name):
-    # Here instead of _utils because we need access to region and cloud
-    # name from the cloud object
-    ret = []
-    for server in servers:
-        ret.append(normalize_server(server, cloud_name, region_name))
-    return ret
-
-
-def normalize_server(server, cloud_name, region_name):
-    server.pop('links', None)
-    server['flavor'].pop('links', None)
-    # OpenStack can return image as a string when you've booted
-    # from volume
-    if str(server['image']) != server['image']:
-        server['image'].pop('links', None)
-
-    server['region'] = region_name
-    server['cloud'] = cloud_name
-
-    az = server.get('OS-EXT-AZ:availability_zone', None)
-    if az:
-        server['az'] = az
-
-    # Ensure volumes is always in the server dict, even if empty
-    server['volumes'] = []
-
-    return server
-
-
 def normalize_keystone_services(services):
     """Normalize the structure of keystone services
 
@@ -203,137 +182,6 @@ def normalize_keystone_services(services):
     return meta.obj_list_to_dict(ret)
 
 
-def normalize_nova_secgroups(groups):
-    """Normalize the structure of nova security groups
-
-    This makes security group dicts, as returned from nova, look like the
-    security group dicts as returned from neutron. This does not make them
-    look exactly the same, but it's pretty close.
-
-    :param list groups: A list of security group dicts.
-
-    :returns: A list of normalized dicts.
-    """
-    ret = [{'id': g['id'],
-            'name': g['name'],
-            'description': g['description'],
-            'security_group_rules': normalize_nova_secgroup_rules(g['rules'])
-            } for g in groups]
-    return meta.obj_list_to_dict(ret)
-
-
-def normalize_nova_secgroup_rules(rules):
-    """Normalize the structure of nova security group rules
-
-    Note that nova uses -1 for non-specific port values, but neutron
-    represents these with None.
-
-    :param list rules: A list of security group rule dicts.
-
-    :returns: A list of normalized dicts.
-    """
-    ret = [{'id': r['id'],
-            'direction': 'ingress',
-            'ethertype': 'IPv4',
-            'port_range_min':
-                None if r['from_port'] == -1 else r['from_port'],
-            'port_range_max':
-                None if r['to_port'] == -1 else r['to_port'],
-            'protocol': r['ip_protocol'],
-            'remote_ip_prefix': r['ip_range'].get('cidr', None),
-            'security_group_id': r['parent_group_id']
-            } for r in rules]
-    return meta.obj_list_to_dict(ret)
-
-
-def normalize_nova_floating_ips(ips):
-    """Normalize the structure of Neutron floating IPs
-
-    Unfortunately, not all the Neutron floating_ip attributes are available
-    with Nova and not all Nova floating_ip attributes are available with
-    Neutron.
-    This function extract attributes that are common to Nova and Neutron
-    floating IP resource.
-    If the whole structure is needed inside shade, shade provides private
-    methods that returns "original" objects (e.g. _nova_allocate_floating_ip)
-
-    :param list ips: A list of Nova floating IPs.
-
-    :returns:
-        A list of normalized dicts with the following attributes::
-
-        [
-          {
-            "id": "this-is-a-floating-ip-id",
-            "fixed_ip_address": "192.0.2.10",
-            "floating_ip_address": "198.51.100.10",
-            "network": "this-is-a-net-or-pool-id",
-            "attached": True,
-            "status": "ACTIVE"
-          }, ...
-        ]
-
-    """
-    ret = [dict(
-        id=ip['id'],
-        fixed_ip_address=ip.get('fixed_ip'),
-        floating_ip_address=ip['ip'],
-        network=ip['pool'],
-        attached=(ip.get('instance_id') is not None and
-                  ip.get('instance_id') != ''),
-        status='ACTIVE'  # In neutrons terms, Nova floating IPs are always
-                         # ACTIVE
-    ) for ip in ips]
-    return meta.obj_list_to_dict(ret)
-
-
-def normalize_neutron_floating_ips(ips):
-    """Normalize the structure of Neutron floating IPs
-
-    Unfortunately, not all the Neutron floating_ip attributes are available
-    with Nova and not all Nova floating_ip attributes are available with
-    Neutron.
-    This function extract attributes that are common to Nova and Neutron
-    floating IP resource.
-    If the whole structure is needed inside shade, shade provides private
-    methods that returns "original" objects (e.g.
-    _neutron_allocate_floating_ip)
-
-    :param list ips: A list of Neutron floating IPs.
-
-    :returns:
-        A list of normalized dicts with the following attributes::
-
-        [
-          {
-            "id": "this-is-a-floating-ip-id",
-            "fixed_ip_address": "192.0.2.10",
-            "floating_ip_address": "198.51.100.10",
-            "network": "this-is-a-net-or-pool-id",
-            "attached": True,
-            "status": "ACTIVE"
-          }, ...
-        ]
-
-    """
-    ret = []
-    for ip in ips:
-        network_id = ip.get('floating_network_id', ip.get('network'))
-        ret.append(dict(
-            id=ip['id'],
-            fixed_ip_address=ip.get('fixed_ip_address'),
-            floating_ip_address=ip['floating_ip_address'],
-            network=network_id,
-            floating_network_id=network_id,
-            port_id=ip.get('port_id'),
-            router_id=ip.get('router_id'),
-            attached=(ip.get('port_id') is not None and
-                      ip.get('port_id') != ''),
-            status=ip.get('status', 'UNKNOWN')
-        ))
-    return meta.obj_list_to_dict(ret)
-
-
 def localhost_supports_ipv6():
     """Determine whether the local host supports IPv6
 
@@ -356,6 +204,7 @@ def normalize_users(users):
                                         user.get('tenantId')),
             domain_id=user.get('domain_id'),
             enabled=user.get('enabled'),
+            description=user.get('description')
         ) for user in users
     ]
     return meta.obj_list_to_dict(ret)
@@ -478,26 +327,6 @@ def normalize_stacks(stacks):
     return stacks
 
 
-def normalize_flavors(flavors):
-    """ Normalize a list of flavor objects """
-    for flavor in flavors:
-        flavor.pop('links', None)
-        flavor.pop('NAME_ATTR', None)
-        flavor.pop('HUMAN_ID', None)
-        flavor.pop('human_id', None)
-        if 'extra_specs' not in flavor:
-            flavor['extra_specs'] = {}
-        ephemeral = flavor.pop('OS-FLV-EXT-DATA:ephemeral', 0)
-        is_public = flavor.pop('os-flavor-access:is_public', True)
-        # Make sure both the extension version and a sane version are present
-        flavor['OS-FLV-EXT-DATA:ephemeral'] = ephemeral
-        flavor['ephemeral'] = ephemeral
-        flavor['os-flavor-access:is_public'] = is_public
-        flavor['is_public'] = is_public
-
-    return flavors
-
-
 def normalize_cluster_templates(cluster_templates):
     """Normalize Magnum cluster_templates."""
     for cluster_template in cluster_templates:
