@@ -17,6 +17,7 @@ import munch
 import ipaddress
 import six
 
+from shade import _log
 from shade import exc
 
 
@@ -80,7 +81,7 @@ def get_server_private_ip(server, cloud=None):
     # Short circuit the ports/networks search below with a heavily cached
     # and possibly pre-configured network name
     if cloud:
-        int_nets = cloud.get_internal_networks()
+        int_nets = cloud.get_internal_ipv4_networks()
         for int_net in int_nets:
             int_ip = get_server_ip(server, key_name=int_net['name'])
             if int_ip is not None:
@@ -122,7 +123,7 @@ def get_server_external_ipv4(cloud, server):
 
     # Short circuit the ports/networks search below with a heavily cached
     # and possibly pre-configured network name
-    ext_nets = cloud.get_external_networks()
+    ext_nets = cloud.get_external_ipv4_networks()
     for ext_net in ext_nets:
         ext_ip = get_server_ip(server, key_name=ext_net['name'])
         if ext_ip is not None:
@@ -294,7 +295,9 @@ def _get_suplemental_addresses(cloud, server):
                 return server['addresses']
             fixed_ip_mapping[address['addr']] = name
     try:
-        if cloud._has_floating_ips():
+        # Don't bother doing this before the server is active, it's a waste
+        # of an API call while polling for a server to come up
+        if cloud._has_floating_ips() and server['status'] == 'ACTIVE':
             for fip in cloud.list_floating_ips():
                 if fip['fixed_ip_address'] in fixed_ip_mapping:
                     fixed_net = fixed_ip_mapping[fip['fixed_ip_address']]
@@ -394,9 +397,24 @@ def get_hostvars_from_server(cloud, server, mounts=None):
     return server_vars
 
 
-def _add_request_id(obj, request_id):
+def _log_request_id(obj, request_id):
+    # Add it, if passed in, even though we're going to pop in a second,
+    # just to make the logic simpler
     if request_id is not None:
         obj['x_openstack_request_ids'] = [request_id]
+
+    request_id = None
+    request_ids = obj.pop('x_openstack_request_ids', None)
+    if request_ids:
+        request_id = request_ids[0]
+    if request_id:
+        log = _log.setup_logging('shade.request_ids')
+        # Log the request id and object id in a specific logger. This way
+        # someone can turn it on if they're interested in this kind of tracing.
+        log.debug("Retreived object %(id)s. Request ID %(request_id)s",
+                  {'id': obj.get('id', obj.get('uuid')),
+                   'request_id': request_id})
+
     return obj
 
 
@@ -417,7 +435,7 @@ def obj_to_dict(obj, request_id=None):
         return obj
     elif hasattr(obj, 'schema') and hasattr(obj, 'validate'):
         # It's a warlock
-        return _add_request_id(warlock_to_dict(obj), request_id)
+        return _log_request_id(warlock_to_dict(obj), request_id)
     elif isinstance(obj, dict):
         # The new request-id tracking spec:
         # https://specs.openstack.org/openstack/nova-specs/specs/juno/approved/log-request-id-mappings.html
@@ -440,7 +458,7 @@ def obj_to_dict(obj, request_id=None):
             continue
         if isinstance(value, NON_CALLABLES) and not key.startswith('_'):
             instance[key] = value
-    return _add_request_id(instance, request_id)
+    return _log_request_id(instance, request_id)
 
 
 def obj_list_to_dict(obj_list, request_id=None):
