@@ -19,22 +19,20 @@ test_inventory
 Functional tests for `shade` inventory methods.
 """
 
-from shade import openstack_cloud
 from shade import inventory
 
-from shade.tests import base
+from shade.tests.functional import base
 from shade.tests.functional.util import pick_flavor, pick_image
 
 
-class TestInventory(base.TestCase):
+class TestInventory(base.BaseFunctionalTestCase):
     def setUp(self):
         super(TestInventory, self).setUp()
         # This needs to use an admin account, otherwise a public IP
         # is not allocated from devstack.
-        self.cloud = openstack_cloud(cloud='devstack-admin')
         self.inventory = inventory.OpenStackInventory()
         self.server_name = 'test_inventory_server'
-        self.nova = self.cloud.nova_client
+        self.nova = self.operator_cloud.nova_client
         self.flavor = pick_flavor(self.nova.flavors.list())
         if self.flavor is None:
             self.assertTrue(False, 'no sensible flavor available')
@@ -42,7 +40,7 @@ class TestInventory(base.TestCase):
         if self.image is None:
             self.assertTrue(False, 'no sensible image available')
         self.addCleanup(self._cleanup_servers)
-        self.cloud.create_server(
+        self.operator_cloud.create_server(
             name=self.server_name, image=self.image, flavor=self.flavor,
             wait=True, auto_ip=True)
 
@@ -52,23 +50,47 @@ class TestInventory(base.TestCase):
                 self.nova.servers.delete(i)
 
     def _test_host_content(self, host):
-        self.assertEquals(host['image']['id'], self.image.id)
+        self.assertEqual(host['image']['id'], self.image.id)
         self.assertNotIn('links', host['image'])
-        self.assertEquals(host['flavor']['id'], self.flavor.id)
+        self.assertEqual(host['flavor']['id'], self.flavor.id)
         self.assertNotIn('links', host['flavor'])
         self.assertNotIn('links', host)
         self.assertIsInstance(host['volumes'], list)
         self.assertIsInstance(host['metadata'], dict)
         self.assertIn('interface_ip', host)
 
+    def _test_expanded_host_content(self, host):
+        self.assertEqual(host['image']['name'], self.image.name)
+        self.assertEqual(host['flavor']['name'], self.flavor.name)
+
     def test_get_host(self):
         host = self.inventory.get_host(self.server_name)
         self.assertIsNotNone(host)
-        self.assertEquals(host['name'], self.server_name)
+        self.assertEqual(host['name'], self.server_name)
         self._test_host_content(host)
+        self._test_expanded_host_content(host)
         host_found = False
         for host in self.inventory.list_hosts():
             if host['name'] == self.server_name:
                 host_found = True
                 self._test_host_content(host)
         self.assertTrue(host_found)
+
+    def test_get_host_no_detail(self):
+        host = self.inventory.get_host(self.server_name, expand=False)
+        self.assertIsNotNone(host)
+        self.assertEqual(host['name'], self.server_name)
+
+        self.assertEqual(host['image']['id'], self.image.id)
+        self.assertNotIn('links', host['image'])
+        self.assertNotIn('name', host['name'])
+        self.assertEqual(host['flavor']['id'], self.flavor.id)
+        self.assertNotIn('links', host['flavor'])
+        self.assertNotIn('name', host['flavor'])
+
+        host_found = False
+        for host in self.inventory.list_hosts(expand=False):
+            if host['name'] == self.server_name:
+                host_found = True
+                self._test_host_content(host)
+        self.assertTrue(host_found)
