@@ -214,6 +214,10 @@ class TestShade(base.TestCase):
             router='123', body={'subnet_id': 'abc'}
         )
 
+    def test_remove_router_interface_missing_argument(self):
+        self.assertRaises(ValueError, self.cloud.remove_router_interface,
+                          {'id': '123'})
+
     @mock.patch.object(shade.OpenStackCloud, 'get_router')
     @mock.patch.object(shade.OpenStackCloud, 'neutron_client')
     def test_update_router(self, mock_client, mock_get):
@@ -259,8 +263,7 @@ class TestShade(base.TestCase):
         self.assertTrue(mock_client.delete_router.called)
 
     @mock.patch.object(shade.OpenStackCloud, 'search_ports')
-    @mock.patch.object(shade.OpenStackCloud, 'neutron_client')
-    def test_list_router_interfaces_no_gw(self, mock_client, mock_search):
+    def test_list_router_interfaces_no_gw(self, mock_search):
         """
         If a router does not have external_gateway_info, do not fail.
         """
@@ -280,9 +283,14 @@ class TestShade(base.TestCase):
         )
         self.assertEqual([], ret)
 
+        # A router can have its external_gateway_info set to None
+        router['external_gateway_info'] = None
+        ret = self.cloud.list_router_interfaces(router,
+                                                interface_type='external')
+        self.assertEqual([], ret)
+
     @mock.patch.object(shade.OpenStackCloud, 'search_ports')
-    @mock.patch.object(shade.OpenStackCloud, 'neutron_client')
-    def test_list_router_interfaces_all(self, mock_client, mock_search):
+    def test_list_router_interfaces_all(self, mock_search):
         internal_port = {'id': 'internal_port_id',
                          'fixed_ips': [
                              ('internal_subnet_id', 'ip_address'),
@@ -306,8 +314,7 @@ class TestShade(base.TestCase):
         self.assertEqual(port_list, ret)
 
     @mock.patch.object(shade.OpenStackCloud, 'search_ports')
-    @mock.patch.object(shade.OpenStackCloud, 'neutron_client')
-    def test_list_router_interfaces_internal(self, mock_client, mock_search):
+    def test_list_router_interfaces_internal(self, mock_search):
         internal_port = {'id': 'internal_port_id',
                          'fixed_ips': [
                              ('internal_subnet_id', 'ip_address'),
@@ -332,8 +339,7 @@ class TestShade(base.TestCase):
         self.assertEqual([internal_port], ret)
 
     @mock.patch.object(shade.OpenStackCloud, 'search_ports')
-    @mock.patch.object(shade.OpenStackCloud, 'neutron_client')
-    def test_list_router_interfaces_external(self, mock_client, mock_search):
+    def test_list_router_interfaces_external(self, mock_search):
         internal_port = {'id': 'internal_port_id',
                          'fixed_ips': [
                              ('internal_subnet_id', 'ip_address'),
@@ -381,8 +387,7 @@ class TestShade(base.TestCase):
         self.assertTrue(mock_client.create_subnet.called)
 
     @mock.patch.object(shade.OpenStackCloud, 'search_networks')
-    @mock.patch.object(shade.OpenStackCloud, 'neutron_client')
-    def test_create_subnet_bad_ip_version(self, mock_client, mock_search):
+    def test_create_subnet_bad_ip_version(self, mock_search):
         '''String ip_versions must be convertable to int'''
         net1 = dict(id='123', name='donald')
         mock_search.return_value = [net1]
@@ -422,8 +427,7 @@ class TestShade(base.TestCase):
         self.assertTrue(mock_client.create_subnet.called)
 
     @mock.patch.object(shade.OpenStackCloud, 'search_networks')
-    @mock.patch.object(shade.OpenStackCloud, 'neutron_client')
-    def test_create_subnet_conflict_gw_ops(self, mock_client, mock_search):
+    def test_create_subnet_conflict_gw_ops(self, mock_search):
         net1 = dict(id='123', name='donald')
         mock_search.return_value = [net1]
         gateway = '192.168.200.3'
@@ -517,8 +521,7 @@ class TestShade(base.TestCase):
         self.assertTrue(mock_client.update_subnet.called)
 
     @mock.patch.object(shade.OpenStackCloud, 'get_subnet')
-    @mock.patch.object(shade.OpenStackCloud, 'neutron_client')
-    def test_update_subnet_conflict_gw_ops(self, mock_client, mock_get):
+    def test_update_subnet_conflict_gw_ops(self, mock_get):
         subnet1 = dict(id='456', name='kooky')
         mock_get.return_value = subnet1
         gateway = '192.168.200.3'
